# Elmer circuitbuilder file is prepared according to templates and instructions in:
# https://github.com/ElmerCSC/elmer-elmag/tree/main/CircuitBuilder
# This python file uses elmer-circuitbuilder library, which can be downloaded using 
# linux command: pip install elmer-circuitbuilder
# and thus, for convenience is run in linux (WSL) environment.
# Additionally, file is case-specific so is prepared in jinja environment.

# Imported Libraries:
import sys
import numpy as np
#import os
import sys
import fileinput

from elmer_circuitbuilder import *                                                 # STEP 1
# -------------------------------------------------------------------------------

def main(argv=None):

    # name output file - do not remove                                             # STEP 2
    output_file = "coil-circuits.definition"

    # initialize circuits: number of circuits - do not remove                      # STEP 3
    c = number_of_circuits(0+3)

    # reference/ground node needed - do not remove.                                # STEP 4
    c[1].ref_node = 1
    c[2].ref_node = 1
    c[3].ref_node = 1

    # ----------------------- Electrical Network Definition ---------------------  # STEP 5
    
    # ----------------------- Wires --------------------- 
      

    # ----------------------- Coils ---------------------                          
    # Components
    
    phase_1 = np.radians(0)

    # Case == Harmonic
    sourceVector = np.exp(phase_1*1j)
    
    sourceValue = 6
    #SourceNo = 1
    #SourceName = 'S'+str(SourceNo)
    # Case == Current loop
    S1 = I("S1", 1,2, sourceValue*sourceVector)
    
    #FEM_component1 = ElmerComponent("Name",pin,pin,componentNumber,[List of bodies -- From SIF file!])
    FEM_Component1 = ElmerComponent("CoilComp1",1,2,1,[1])

    # Define coil type: massive, stranded, foil
    # and dimension related features if needed (closed, open)

    # Case == Current loop
    ##FEM_Component1.stranded(turnNo, Resistance)
    FEM_Component1.stranded(10000, 0)
    FEM_Component1.isClosed()
    
    FEM_Component1.is3D()

    # store components in array components = [comp1, comp2,...] - do not remove    # STEP 6
    c[1].components.append([S1,FEM_Component1])  
    # Components
    
    phase_2 = np.radians(60)

    # Case == Harmonic
    sourceVector = np.exp(phase_2*1j)
    
    sourceValue = 6
    #SourceNo = 2
    #SourceName = 'S'+str(SourceNo)
    # Case == Current loop
    S2 = I("S2", 1,2, sourceValue*sourceVector)
    
    #FEM_component2 = ElmerComponent("Name",pin,pin,componentNumber,[List of bodies -- From SIF file!])
    FEM_Component2 = ElmerComponent("CoilComp2",1,2,2,[2])

    # Define coil type: massive, stranded, foil
    # and dimension related features if needed (closed, open)

    # Case == Current loop
    ##FEM_Component1.stranded(turnNo, Resistance)
    FEM_Component2.stranded(10000, 0)
    FEM_Component2.isClosed()
    
    FEM_Component2.is3D()

    # store components in array components = [comp1, comp2,...] - do not remove    # STEP 6
    c[2].components.append([S2,FEM_Component2])  
    # Components
    
    phase_3 = np.radians(120)

    # Case == Harmonic
    sourceVector = np.exp(phase_3*1j)
    
    sourceValue = 6
    #SourceNo = 3
    #SourceName = 'S'+str(SourceNo)
    # Case == Current loop
    S3 = I("S3", 1,2, sourceValue*sourceVector)
    
    #FEM_component3 = ElmerComponent("Name",pin,pin,componentNumber,[List of bodies -- From SIF file!])
    FEM_Component3 = ElmerComponent("CoilComp3",1,2,3,[3])

    # Define coil type: massive, stranded, foil
    # and dimension related features if needed (closed, open)

    # Case == Current loop
    ##FEM_Component1.stranded(turnNo, Resistance)
    FEM_Component3.stranded(10000, 0)
    FEM_Component3.isClosed()
    
    FEM_Component3.is3D()

    # store components in array components = [comp1, comp2,...] - do not remove    # STEP 6
    c[3].components.append([S3,FEM_Component3])  
                               

    # ---------------------------------------------------------------------------

    # generate elmer circuit.definitions - do not remove / do not edit             # STEP 7
    generate_elmer_circuits(c, output_file)   
         
    # Add time function for transient case
    # run ElmerSolver
    #os.system("ElmerSolver")  
    
    return 0


if __name__ == "__main__":
    sys.exit(main() or 0)

